' CORRELATED MULTIVARIATE RANDOM NORMAL SCORES GENERATOR

' This program functions by first reducing a specified symmetric correlation

' matrix R into the product of a lower triangular matrix C and its

' transpose, R = C'C.  Then, a random normal vector Y ~ N(0, I) is generated

' using the Box and Muller (1958) algorithms.  Finally, the vector X is

' obtained by multiplying the generated vector Y by the lower triangular

' matrix C (i.e., X = YC ~ N(0, R)).  The computation of C for a given R is

' achieved by means of the Square Root Method (e.g., Scheuer and Stoller,

' 1962).

'

'Citation:

'  Aguinis, H. (1994).  A QuickBASIC program for generating correlated

'    multivariate random normal scores.  Educational and Psychological

'    Measurement, 54, 687-689.

'

'Address:

'  University of Colorado at Denver

'  Graduate School of Business Administration

'  Campus Box 165; P.O. Box 173364

'  Denver, CO 80217-3364

'

' E-mail: Herman.Aguinis@cudenver.edu (Internet)

' WWW: http://www.cudenver.edu/~haguinis



'************************************************************************

DECLARE SUB DisplayHeader ()

DECLARE SUB InitializeVariables ()

DECLARE SUB WriteStats (Sample!, NumberOfSamples!, N!, NumberOfCases!)

DECLARE SUB WriteCaseLowAcc (N!, CaseNumber!)

DECLARE SUB WriteCaseHighAcc (N!, CaseNumber!)

DECLARE SUB DisplayStats (Sample!, NumberOfSamples!, N!, NumberOfCases!)

DECLARE SUB ComputeStats (N!, NumberOfCases!)

DECLARE SUB GenerateXvalues (N!, NumberOfCases!)

DECLARE SUB GenerateMultivariate (N!, NumberOfCases!)

DECLARE SUB GenerateMatrixC (N!)

DECLARE SUB GetCovariances (N!)

DECLARE SUB GetUserInformation (N!, NumberOfCases!, NumberOfSamples!)

DECLARE SUB GenerateYvalues (N!)



'************************************************************************

'Array Declarations

'

'************************************************************************

     DIM SHARED Covar!(10, 10)       'Covariance relations matrix

     DIM SHARED c!(10, 10)           'The matrix C

     DIM SHARED X!(10, 1000)         'The X vector

     DIM SHARED y!(10)               'The Y vector



     'Arrays needed to compute statistics for the generated data

     DIM SHARED SumX!(10)

     DIM SHARED SumXsquare!(10)

     DIM SHARED SumXcross!(10, 10)

     DIM SHARED Xbar!(10)

     DIM SHARED SumXdevSqr!(10)

     DIM SHARED StdDevOfX!(10)

     DIM SHARED CorrX!(10, 10)



'************************************************************************

'  Main Body of Program

'

'************************************************************************

     CLS                                  'Clear the screen

     RANDOMIZE TIMER



     CALL DisplayHeader

     CALL GetUserInformation(N, NumberOfCases, NumberOfSamples)

     CALL GetCovariances(N)

     CALL GenerateMatrixC(N)

     LOCATE 1, 60

     COLOR 31

     PRINT " *** Please wait *** "

     COLOR 7

    

     FOR Sample = 1 TO NumberOfSamples

          CALL InitializeVariables

          CALL GenerateMultivariate(N, NumberOfCases)

          CALL ComputeStats(N, NumberOfCases)

          CALL DisplayStats(Sample, NumberOfSamples, N, NumberOfCases)

          CALL WriteStats(Sample, NumberOfSamples, N, NumberOfCases)

    

     NEXT Sample



    LOCATE 1, 60: PRINT "                    "

END     'Program



'************************************************************************

'Compute Statistics Module

'

'    This module computes the sample-based means, standard deviations,

'    and correlations

'

'

'    Variables Used in Module

'    ------------------------

'         i, j                    Local to Module

'         N, NumberOfCases        Parameters to Module

'         CorrX ()                Global to Module

'         StdDevOfX ()            Global to Module

'         SumX ()                 Global to Module

'         SumXcross ()            Global to Module

'         SumXdevSqr ()           Global to Module

'         SumXsquare()            Global to Module

'         Xbar ()                 Global to Module

'

'    Variables Modified by this Module

'    ---------------------------------

'         CorrX ()

'         StdDevOfX ()

'         SumX ()

'         SumXcross ()

'         SumXdevSqr ()

'         SumXsquare()

'

'************************************************************************

SUB ComputeStats (N, NumberOfCases)

    

     FOR CaseNumber = 1 TO NumberOfCases

          FOR i = 1 TO N

               SumX(i) = SumX(i) + X(i, CaseNumber)

               SumXsquare(i) = SumXsquare(i) + X(i, CaseNumber) ^ 2

               FOR j = (i + 1) TO N

                    SumXcross(i, j) = SumXcross(i, j) + (X(i, CaseNumber) * X(j, CaseNumber))



               NEXT j

          NEXT i

     NEXT CaseNumber



     FOR i = 1 TO N

          Xbar(i) = SumX(i) / NumberOfCases

     NEXT i



     'Initialize SumXdevSqr array

     FOR i = 1 TO N: SumXdevSqr(i) = 0!: NEXT i



     FOR CaseNumber = 1 TO NumberOfCases

          FOR i = 1 TO N

               SumXdevSqr(i) = (X(i, CaseNumber) - Xbar(i)) ^ 2 + SumXdevSqr(i)

          NEXT i

     NEXT CaseNumber



     FOR i = 1 TO N

          StdDevOfX(i) = SQR(SumXdevSqr(i) / (NumberOfCases - 1))

     NEXT i



     FOR i = 1 TO N

          FOR j = (i + 1) TO N

               CorrX(i, j) = (NumberOfCases * SumXcross(i, j) - SumX(i) * SumX(j)) / (SQR(NumberOfCases * SumXsquare(i) - SumX(i) ^ 2) * SQR(NumberOfCases * SumXsquare(j) - SumX(j) ^ 2))

          NEXT j

     NEXT i







END SUB



SUB DisplayHeader

PRINT : PRINT : PRINT

PRINT : PRINT

PRINT "           CORRELATED MULTIVARIATE RANDOM NORMAL SCORES GENERATOR ";

PRINT

PRINT "                              Herman Aguinis"

PRINT "                Graduate School of Business Administration"

PRINT "                     University of Colorado at Denver"

PRINT : PRINT "                 Internet: Herman.Aguinis@cudenver.edu"

PRINT "                 WWW: http://www.cudenver.edu/~haguinis"

PRINT : PRINT

PRINT "        Citation:"

PRINT "        Aguinis, H. (1994).  A QuickBASIC program for generating correlated"

PRINT "        multivariate random normal scores.  Educational and Psychological"

PRINT "        Measurement, 54, 687-689."

PRINT ""

PRINT : PRINT : PRINT : PRINT : PRINT : PRINT

PRINT " Press any key to continue "

PRINT

ak$ = INPUT$(1)

CLS

END SUB



'************************************************************************

'Display Statistics Module

'

'    This module displays the statistics as they are being computed.

'

'    Variables Used in Module

'    ------------------------

'         i, j                     Local to Module

'         N, NumberOfCases         Parameters to Module

'         StdDevOfX ()             Global to Module

'         Xbar ()                  Global to Module

'         CorrX ()                 Global to Module

'

'    Variables Modified by this Module

'    ---------------------------------

'         none

'

'************************************************************************

SUB DisplayStats (Sample, NumberOfSamples, N, NumberOfCases)

     CLS

     LOCATE 1, 60

     COLOR 31

     PRINT " *** Please wait *** "

     COLOR 7



     LOCATE 2, 1

     PRINT "Sample"; Sample; " of "; NumberOfSamples

     PRINT "Sample Size = "; NumberOfCases

    

     FOR i = 1 TO N

       PRINT "Mean X"; i; " = "; : PRINT USING "##.######"; Xbar(i)

     NEXT i



     FOR i = 1 TO N

          PRINT "Standard Deviation X"; i; " = "; StdDevOfX(i)

     NEXT i



     FOR i = 1 TO N

      FOR j = (i + 1) TO N

       PRINT "r X"; i; "X"; j; " = "; : PRINT USING "##.######"; CorrX(i, j)

      NEXT j

     NEXT i

     

END SUB



'************************************************************************

'Generate Matrix C Module

'

'    This module creates the matrix C via the Square Root Method

'

'

'    Variables Used in Module

'    ------------------------

'         i, j, k                       Local to Module

'         Csum                          Local to Module

'         N                             Parameter to Module

'         C ()                          Global to Module

'         Covar ()                      Global to Module

'

'    Variables Modified by this Module

'    ---------------------------------

'         C ()

'

'************************************************************************

SUB GenerateMatrixC (N)

     FOR i = 1 TO N

          FOR j = 1 TO N

              

               'CASE #1

               IF (j = 1) THEN

                    c(i, j) = Covar(i, 1) / SQR(Covar(1, 1))

              

               'CASE #2

               ELSEIF ((i = j) AND (i <> 1)) THEN

                    Csum = 0



                    FOR k = 1 TO (i - 1)

                    Csum = Csum + c(i, k) ^ 2



                    NEXT k



                    c(i, j) = SQR(Covar(i, j) - Csum)

                  

               'CASE #3

               ELSEIF ((j < i) AND (i <> 1) AND (j <> 1)) THEN

                    Csum = 0



                    FOR k = 1 TO (j - 1)

                         Csum = Csum + c(i, k) * c(j, k)



                    NEXT k



                    c(i, j) = (Covar(i, j) - Csum) / c(j, j)

                  

               'CASE #4

               ELSE

                    c(i, j) = 0!



               END IF



          NEXT j



     NEXT i

    

END SUB



'************************************************************************

'Generate Multivariate Subroutine

'

'    This module controls the sequence for caculating the necessary

'    values for the matrix C, the Y vector, and the X vector.

'

'

'    Variables Used in Module

'    ------------------------

'         NumberOfCases

'

'

'    Variables Modified by the Module

'    --------------------------------

'         none

'

'************************************************************************

SUB GenerateMultivariate (N, NumberOfCases)



     FOR CaseNumber = 1 TO NumberOfCases

          CALL GenerateYvalues(N)

          CALL GenerateXvalues(N, CaseNumber)

         

          IF N <= 7 THEN      'Number of variables 1-7 write higher accuracy

               CALL WriteCaseHighAcc(N, CaseNumber)

         

          ELSE                'Number of variables 7-10 write lower accuracy

               CALL WriteCaseLowAcc(N, CaseNumber)

         

          END IF



     NEXT CaseNumber



END SUB



'************************************************************************

'Generate X Vector Module

'

'    This module generates the X vector for a given CaseNumber.

'

'    Variables Used in Module

'    ------------------------

'         i, j                          Local to Module

'         N, CaseNumber                 Parameter to Module

'         C ()                          Global to Module

'         Covar ()                      Global to Module

'         X ()                          Global to Module

'

'    Variables Modified by this Module

'    ---------------------------------

'         X ()

'

'************************************************************************

SUB GenerateXvalues (N, CaseNumber)

     FOR i = 1 TO N

        X(i, CaseNumber) = 0!



          FOR j = 1 TO i

               X(i, CaseNumber) = X(i, CaseNumber) + c(i, j) * y(j)

         

          NEXT j



     NEXT i



END SUB



'************************************************************************

'Generate Y Vector Module

'

'    This module generates the Y vector to be used in the calculation

'    of the X vector.

'

'    Variables Used in Module

'    ------------------------

'         i                             Local to Module

'         U1, U2                        Local to Module

'         pi                            Local Constant to Module

'         N                             Parameter to Module

'         Y ()                          Global to Module

'

'    Variables Modified by this Module

'    ---------------------------------

'         Y ()

'

'************************************************************************

SUB GenerateYvalues (N)

      pi = 3.1415926535898#

    

     i = 1

     DO WHILE i <= N

          U1 = RND

          U2 = RND





          y(i) = SQR(-2! * (LOG(U1))) * COS(2! * pi * U2)

          i = i + 1



          IF i <= N THEN

               y(i) = SQR(-2! * (LOG(U1))) * SIN(2! * pi * U2)

          ELSE :

          i = i + 1



          END IF

        

     LOOP

END SUB



'************************************************************************

'Get Covariances Module

'

'    This module gets the population covariances (correlations in this case)

'    from the user.

'

'    Variables Used in Module

'    ------------------------

'         i, j                          Local to Module

'         N                             Parameter to Module

'         Covar ()                      Global to Module

'

'    Variables Modified by the Module

'    --------------------------------

'         Covar ()

'

'************************************************************************

SUB GetCovariances (N)



     i = 1

     DO WHILE i <= N

          j = i

          Covar(i, j) = 1

          j = j + 1



          DO WHILE j <= N

          



c:       PRINT "Population Correlation between X"; i; " and  X"; j; ":  ";

               INPUT Covar(i, j)

               IF Covar(i, j) > 1! OR Covar(i, j) < -1! THEN GOTO c:

               Covar(j, i) = Covar(i, j)

               j = j + 1



          LOOP 'End DO WHILE j <= N

         

          i = i + 1

    

     LOOP 'End DO WHILE i <= N



END SUB



'************************************************************************

'Get User Information Module

'

'    This module gets the number of variables (N), the

'    number of cases (NumberOfCases), and the number of samples (NumberOfSamples)

'

'    Variables Used in Module

'    ------------------------

'         N, NumberOfCases              Parameters to Module

'

'    Variables Modified by the Module

'    --------------------------------

'         N, NumberOfCases

'

'************************************************************************

SUB GetUserInformation (N, NumberOfCases, NumberOfSamples)

LOCATE 3, 1

        INPUT "Enter the number of samples  :  "; NumberOfSamples

a:     INPUT "Enter the number of variables:  "; N

       IF N > 10 THEN GOTO a:

b:       INPUT "Enter the number of cases:      "; NumberOfCases

       IF NumberOfCases > 1000 THEN GOTO b:

        PRINT

        

END SUB



SUB InitializeVariables

   

     FOR i = 1 TO 10

          SumX!(i) = 0!

          SumXsquare!(i) = 0!

          Xbar!(i) = 0!

          SumXdevSqr!(i) = 0!

          StdDevOfX!(i) = 0!

          y!(i) = 0!



          FOR j = 1 TO 10

               SumXcross!(i, j) = 0!

               CorrX!(i, j) = 0!

          NEXT j



     NEXT i

     

     FOR i = 1 TO 10

          FOR j = 1 TO 1000

               X!(i, j) = 0!

          NEXT j

     NEXT i



END SUB



'************************************************************************

' Write Case to File with High Accuracy Module

'

'    This module writes out the given CASE for a generated X vector.

'    The values written are to 7 decimal places.

'

'    Variables Used in Module

'    ------------------------

'         i                             Local to module

'         CaseNumber                    Parameter to module

'         N                             Parameter to module

'         X                             Global to module

'

'    Variables Modified by the Module

'    --------------------------------

'         none

'

'************************************************************************

SUB WriteCaseHighAcc (N, CaseNumber)

     OPEN "data.out" FOR APPEND AS #1

   

     FOR i = 1 TO N

          PRINT #1, USING "##.#######"; X(i, CaseNumber);

          PRINT #1, " ";

     NEXT i



     PRINT #1, ""



     CLOSE #1



END SUB



'************************************************************************

' Write Case to File with Low Accuracy Module

'

'    This module writes out the given CASE for a generated X vector.

'    The values written are to 3 decimal places.

'

'    Variables Used in Module

'    ------------------------

'         i                             Local to module

'         CaseNumber                    Parameter to module

'         N                             Parameter to module

'         X                             Global to module

'

'    Variables Modified by the Module

'    --------------------------------

'         none

'

'************************************************************************

SUB WriteCaseLowAcc (N, CaseNumber)

     OPEN "data.out" FOR APPEND AS #1

    

     FOR i = 1 TO N

          PRINT #1, USING "##.###"; X(i, CaseNumber);

          PRINT #1, " ";

     NEXT i



     PRINT #1, ""



     CLOSE #1



END SUB



'************************************************************************

' Write Statistics to File Module

'

'    This module writes the statistics out each sample

'

'    Variables Used in Module

'    ------------------------

'         i                             Local to module

'         j                             Local to module

'         N                             Parameter to module

'         NumberOfCases                 Parameter to module

'         NumberOfSamples               Parameter to module

'         Sample                        Parameter to module

'         CorrX                         Global to module

'         StdDevOfX                     Global to module

'         Xbar                          Global to module

'

'    Variables Modified by the Module

'    --------------------------------

'         none

'

'************************************************************************

SUB WriteStats (Sample, NumberOfSamples, N, NumberOfCases)

     OPEN "stats.out" FOR APPEND AS #1

     PRINT #1, "Sample "; Sample; " of "; NumberOfSamples

     PRINT #1, "Sample Size = "; NumberOfCases

     PRINT #1, ""



     FOR i = 1 TO N

     PRINT #1, "Mean X"; i; " = ";

     PRINT #1, USING "#.#######"; Xbar(i)

     NEXT i

     PRINT #1, ""



     FOR i = 1 TO N

          PRINT #1, "Standard Deviation X"; i; " = "; StdDevOfX(i)

     NEXT i

     PRINT #1, ""



     FOR i = 1 TO N

          FOR j = (i + 1) TO N

          PRINT #1, "r X"; i; "X"; j; " = ";

          PRINT #1, USING "#.#######"; CorrX(i, j)

          NEXT j

     NEXT i

   

     PRINT #1, ""

     PRINT #1, "*******************************************************"

     PRINT #1, ""

     CLOSE #1

    

END SUB



